 /*=========================================================================
00002 
00003   Program:   ParaView
00004   Module:    $RCSfile: vtkPVReaderModule.h,v $
00005 
00006   Copyright (c) Kitware, Inc.
00007   All rights reserved.
00008   See Copyright.txt or http://www.paraview.org/HTML/Copyright.html for details.
00009 
00010      This software is distributed WITHOUT ANY WARRANTY; without even
00011      the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
00012      PURPOSE.  See the above copyright notice for more information.
00013 
00014 =========================================================================*/
 #ifndef __vtkPVReaderModule_h
 #define __vtkPVReaderModule_h
 
 #include "vtkPVSource.h"
 
 class vtkPVFileEntry;
 //BTX
 template <class value>
 class vtkVector;
 template <class value>
 class vtkVectorIterator;
 //ETX
 
 class VTK_EXPORT vtkPVReaderModule : public vtkPVSource
 {
 public:
   static vtkPVReaderModule* New();
   vtkTypeRevisionMacro(vtkPVReaderModule, vtkPVSource);
   void PrintSelf(ostream& os, vtkIndent indent);
     
   virtual void CreateProperties();
 
   virtual int CanReadFile(const char* fname);
 
 
   virtual int Initialize(const char* fname, vtkPVReaderModule*& newModule);
   virtual int Finalize  (const char* fname);
   virtual int ReadFileInformation(const char* fname);
 
   void AddExtension(const char*);
   
   vtkIdType GetNumberOfExtensions();
   
   const char* GetExtension(vtkIdType i);
 
   const char* RemovePath(const char* fname);
 
 
   vtkSetMacro(AcceptAfterRead, int);
   vtkGetMacro(AcceptAfterRead, int);
 
 
   vtkGetObjectMacro(FileEntry, vtkPVFileEntry);
 
   virtual void SaveState(ofstream *file);
 
   void AddPVFileEntry(vtkPVFileEntry *pvw);
   
   virtual int GetNumberOfTimeSteps();
   
   virtual void SetRequestedTimeStep(int);
  
   vtkPVWidget* GetTimeStepWidget();
  
 protected:
   vtkPVReaderModule();
   ~vtkPVReaderModule();
 
   const char* ExtractExtension(const char* fname);
 
   vtkPVFileEntry* FileEntry;
   int AcceptAfterRead;
 
   virtual int FinalizeInternal(const char* fname, 
                                int accept);
 
   void SetReaderFileName(const char* fname);
   
 //BTX
   vtkVector<const char*>* Extensions;
   vtkVectorIterator<const char*>* Iterator;
 //ETX
 
   int CloneAndInitialize(int makeCurrent, vtkPVReaderModule*& clone);
 
   int PackFileEntry;
   int AddFileEntry;
 
 private:
   vtkPVReaderModule(const vtkPVReaderModule&); // Not implemented
   void operator=(const vtkPVReaderModule&); // Not implemented
 };
#endif