
//BIOIMAGESUITE_LICENSE  ---------------------------------------------------------------------------------
//BIOIMAGESUITE_LICENSE  This file is part of the BioImage Suite Software Package.
//BIOIMAGESUITE_LICENSE  
//BIOIMAGESUITE_LICENSE  X. Papademetris, M. Jackowski, N. Rajeevan, R.T. Constable, and L.H
//BIOIMAGESUITE_LICENSE  Staib. BioImage Suite: An integrated medical image analysis suite, Section
//BIOIMAGESUITE_LICENSE  of Bioimaging Sciences, Dept. of Diagnostic Radiology, Yale School of
//BIOIMAGESUITE_LICENSE  Medicine, http://www.bioimagesuite.org.
//BIOIMAGESUITE_LICENSE  
//BIOIMAGESUITE_LICENSE  This program is free software; you can redistribute it and/or
//BIOIMAGESUITE_LICENSE  modify it under the terms of the GNU General Public License version 2
//BIOIMAGESUITE_LICENSE  as published by the Free Software Foundation.
//BIOIMAGESUITE_LICENSE  
//BIOIMAGESUITE_LICENSE  This program is distributed in the hope that it will be useful,
//BIOIMAGESUITE_LICENSE  but WITHOUT ANY WARRANTY; without even the implied warranty of
//BIOIMAGESUITE_LICENSE  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//BIOIMAGESUITE_LICENSE  GNU General Public License for more details.
//BIOIMAGESUITE_LICENSE  
//BIOIMAGESUITE_LICENSE  You should have received a copy of the GNU General Public License
//BIOIMAGESUITE_LICENSE  along with this program; if not, write to the Free Software
//BIOIMAGESUITE_LICENSE  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//BIOIMAGESUITE_LICENSE  See also  http://www.gnu.org/licenses/gpl.html
//BIOIMAGESUITE_LICENSE  
//BIOIMAGESUITE_LICENSE  If this software is modified please retain this statement and add a notice
//BIOIMAGESUITE_LICENSE  that it had been modified (and by whom).  
//BIOIMAGESUITE_LICENSE  
//BIOIMAGESUITE_LICENSE  -----------------------------------------------------------------------------------







/*=========================================================================

  Program:   vtkpxcontrib library file
  Module:    $RCSfile: vtkpxAnalyzeImageSource.h,v $
  Language:  C++
  Date:      $Date: 2002/08/09 20:40:16 $
  Version:   $Revision: 1.2 $


Copyright (c) 1995-2001 Xenios Papademetris papad@noodle.med.yale.edu
All rights reserved.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

=========================================================================*/
// .NAME vtkpxAnalyzeImageSource - reads analyze (or binary) 3d images 
// .SECTION Description
// This class reads analyze (.hdr,.img) 3D/4D images from files
// By manually setting the header information, it can also read binary images stored in a single file
// The output of this class is vtkStructuredPoints (either uchar,short,float)
// It can handle byteswapping automatically in the case of analyze image input
// and can read gzipped compressed files directly (automatically adds .gz suffix if needed)
// <br> <br> <EM>Copyright (c) 1995-2001 Xenios Papademetris <a href="mailto:papad@noodle.med.yale.edu">papad@noodle.med.yale.edu</a></EM>
// .SECTION Caveats
// This uses the pxutil library -- for analyze header 
//
// .SECTION See Also
//  vtkpxAnalyzeImageWriter vtkVolume16Reader


#ifndef __vtkpxAnalyzeImageSource
#define __vtkpxAnalyzeImageSource

#include <stdio.h>
#include "vtkStructuredPoints.h"
#include "vtkStructuredPointsReader.h"
#include "pxutil.h"

class vtkDataArray;

class vtkpxAnalyzeImageSource : public vtkStructuredPointsReader
{
public:
  vtkTypeMacro(vtkpxAnalyzeImageSource,vtkStructuredPointsReader);
  static vtkpxAnalyzeImageSource* New();

  // Description:
  // Convenience Routine for analyze images -- simply use Load("fname.hdr")
  virtual int  Load(const char* fname);

  // Description:
  // Reads Header first set using SetFilePrefix
  virtual int  ReadHeaderFile();

  // Description:
  // Set HeaderInfo manually for binary files
  virtual void SetHeaderInfo(int width,int height,int depth,int swap,int bytes);
  virtual void SetHeaderInfo(int width,int height,int depth,int nframes,int swap,int bytes);
  virtual void SetVoxelDimensions(float vox1,float vox2,float vox3);

  // Description:
  // Set Number of TimeFrames for binary files (in case of 4D Images)
  virtual void SetNumTimeFrames(int timeframes);

  // Description:
  // In case of 4D Images returns number of timeframes
  virtual int  GetNumTimeFrames();

  // Description:
  // In case of 4D Images returns number of slices/frame
  virtual int  GetNumSlices();

  // Description:
  // Get the orientation from analyze header 0=axial 1=coronal 2=saggital (default=0) 3=polar (ultrasound)
  vtkGetMacro(Orientation,int);
  vtkSetMacro(Orientation,int);

  // Description:
  // Get the file type 1=bit 2=short 3=unsigned int 4=float
  vtkGetMacro(FileType,int);
  vtkSetClampMacro(FileType,int,1,4);


  // Description:
  // Force the output scalars to have type short
  vtkGetMacro(ForceOutputToShort,int);
  vtkSetMacro(ForceOutputToShort,int);
  vtkBooleanMacro(ForceOutputToShort,int);

  // Description:
  // SaveMultiple Frames As Components 
  vtkGetMacro(MultipleFramesAsComponents,int);
  vtkSetClampMacro(MultipleFramesAsComponents,int,0,1);
  vtkBooleanMacro(MultipleFramesAsComponents, int);

  // Description:
  // Specify the dimensions for the data.
  vtkSetVector2Macro(DataDimensions,int);
  vtkGetVectorMacro(DataDimensions,int,2);

  // Description:
  // Specify the number of bytes to seek over at start of image.
  vtkSetMacro(HeaderSize,int);
  vtkGetMacro(HeaderSize,int);

  // Description:
  // Specify file prefix for the image file(s).
  vtkSetStringMacro(FilePrefix);
  vtkGetStringMacro(FilePrefix);

  // Description:
  // The sprintf format used to build filename from FilePrefix and number.
  vtkSetStringMacro(FilePattern);
  vtkGetStringMacro(FilePattern);

  // Description:
  // Set the range of files to read.
  vtkSetVector2Macro(ImageRange,int);
  vtkGetVectorMacro(ImageRange,int,2);

  // Description:
  // Specify the spacing for the data.
  vtkSetVector3Macro(DataSpacing,double);
  vtkGetVectorMacro(DataSpacing,double,3);

  // Description:
  // Specify the origin for the data.
  vtkSetVector3Macro(DataOrigin,double);
  vtkGetVectorMacro(DataOrigin,double,3);

  // Description:
  // Turn on/off byte swapping.
  vtkSetMacro(SwapBytes,int);
  vtkGetMacro(SwapBytes,int);
  vtkBooleanMacro(SwapBytes,int);


protected:
  //BTX
  int read_volume_ok;
  int FileType;
  int AnalyzeMode;
  int NumTimeFrames;
  int Orientation;
  int ForceOutputToShort;
  int MultipleFramesAsComponents;

  int  DataDimensions[2];
  int   SwapBytes;
  int   HeaderSize;


  char *FilePrefix;
  char *FilePattern;
  int ImageRange[2];
  double DataSpacing[3];
  double DataOrigin[3];


  vtkpxAnalyzeImageSource();
  virtual ~vtkpxAnalyzeImageSource();

  virtual void Execute();
  virtual void ExecuteInformation();


  virtual vtkDataArray* ReadVolume(int FirstImage, int LastImage);
  int ReadBinaryImage(gzFile fp,  short *pixels, int xsize, int ysize,  
		      int skip, int swapBytes);

  int ReadCharBinaryImage(gzFile fp,  char *pixels, int xsize, int ysize,  
		      int skip);

  int ReadIntBinaryImage(gzFile fp,  int *pixels, int xsize, int ysize,  
			 int skip, int swapBytes);
  
  int ReadFloatBinaryImage(gzFile fp,  float *pixels, int xsize, int ysize,  
			   int skip, int swapBytes);
  //ETX
};

#endif


